@@ -1,9 +1,9 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 2001-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -28,32 +28,9 @@
                                   I couldn't give it up because by that
                                   time I was too famous.''
                                             -- Unknown                */
-#include "mod_ssl.h"
+#include "ssl_private.h"
 
 static void ssl_configure_env(request_rec *r, SSLConnRec *sslconn);
- 
-/* Perform a speculative (and non-blocking) read from the connection
- * filters for the given request, to determine whether there is any
- * pending data to read.  Return non-zero if there is, else zero. */
-static int has_buffered_data(request_rec *r) 
-{
-    apr_bucket_brigade *bb;
-    apr_off_t len;
-    apr_status_t rv;
-    int result;
-
-    bb = apr_brigade_create(r->pool, r->connection->bucket_alloc);
-
-    rv = ap_get_brigade(r->connection->input_filters, bb, AP_MODE_SPECULATIVE,
-                        APR_NONBLOCK_READ, 1);
-    result = rv == APR_SUCCESS
-        && apr_brigade_length(bb, 1, &len) == APR_SUCCESS
-        && len > 0;
-
-    apr_brigade_destroy(bb);
-
-    return result;
-}
 
 /*
  *  Post Read Request Handler
@@ -97,7 +74,7 @@ int ssl_hook_ReadReq(request_rec *r)
          * with using SSL on this request.
          */
         sslconn->non_ssl_request = 0;
-        
+
 
         return HTTP_BAD_REQUEST;
     }
@@ -204,7 +181,17 @@ int ssl_hook_Access(request_rec *r)
      * Support for SSLRequireSSL directive
      */
     if (dc->bSSLRequired && !ssl) {
-        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, 
+        if (sc->enabled == SSL_ENABLED_OPTIONAL) {
+            /* This vhost was configured for optional SSL, just tell the
+             * client that we need to upgrade.
+             */
+            apr_table_setn(r->err_headers_out, "Upgrade", "TLS/1.0, HTTP/1.1");
+            apr_table_setn(r->err_headers_out, "Connection", "Upgrade");
+
+            return HTTP_UPGRADE_REQUIRED;
+        }
+
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
                       "access to %s failed, reason: %s",
                       r->filename, "SSL connection required");
 
@@ -215,14 +202,11 @@ int ssl_hook_Access(request_rec *r)
     }
 
     /*
-     * Check to see whether SSL is in use; if it's not, then no
-     * further access control checks are relevant.  (the test for
-     * sc->enabled is probably strictly unnecessary)
+     * Check to see if SSL protocol is on
      */
-    if (!sc->enabled || !ssl) {
+    if (!((sc->enabled == SSL_ENABLED_TRUE) || (sc->enabled == SSL_ENABLED_OPTIONAL) || ssl)) {
         return DECLINED;
     }
-
     /*
      * Support for per-directory reconfigured SSL connection parameters.
      *
@@ -230,8 +214,8 @@ int ssl_hook_Access(request_rec *r)
      * reconfigured parameter suite. But Apache's internal API processing
      * makes our life very hard here, because when internal sub-requests occur
      * we nevertheless should avoid multiple unnecessary SSL handshakes (they
-     * require extra network I/O and especially time to perform). 
-     * 
+     * require extra network I/O and especially time to perform).
+     *
      * But the optimization for filtering out the unnecessary handshakes isn't
      * obvious and trivial.  Especially because while Apache is in its
      * sub-request processing the client could force additional handshakes,
@@ -385,7 +369,7 @@ int ssl_hook_Access(request_rec *r)
      * override of SSLVerifyClient
      *
      * We force a renegotiation if the reconfigured/new verify type is
-     * stronger than the currently active verify type. 
+     * stronger than the currently active verify type.
      *
      * The order is: none << optional_no_ca << optional << require
      *
@@ -514,7 +498,7 @@ int ssl_hook_Access(request_rec *r)
      * handshake immediately; once the SSL library moves to the
      * "accept" state, it will reject the SSL packets which the client
      * is sending for the request body.
-     * 
+     *
      * To allow authentication to complete in this auth hook, the
      * solution used here is to fill a (bounded) buffer with the
      * request body, and then to reinject that request body later.
@@ -625,50 +609,21 @@ int ssl_hook_Access(request_rec *r)
         else {
             request_rec *id = r->main ? r->main : r;
 
-            /* Additional mitigation for CVE-2009-3555: At this point,
-             * before renegotiating, an (entire) request has been read
-             * from the connection.  An attacker may have sent further
-             * data to "prefix" any subsequent request by the victim's
-             * client after the renegotiation; this data may already
-             * have been read and buffered.  Forcing a connection
-             * closure after the response ensures such data will be
-             * discarded.  Legimately pipelined HTTP requests will be
-             * retried anyway with this approach. */
-            if (has_buffered_data(r)) {
-                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
-                              "insecure SSL re-negotiation required, but "
-                              "a pipelined request is present; keepalive "
-                              "disabled");
-                r->connection->keepalive = AP_CONN_CLOSE;
-            }
-
-            /* Perform a full renegotiation. */
-            ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
-                          "Performing full renegotiation: complete handshake "
-                          "protocol (%s support secure renegotiation)",
-#if defined(SSL_get_secure_renegotiation_support)
-                          SSL_get_secure_renegotiation_support(ssl) ? 
-                          "client does" : "client does not"
-#else
-                          "server does not"
-#endif
-                );
+            /* do a full renegotiation */
+            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+                         "Performing full renegotiation: "
+                         "complete handshake protocol");
 
             SSL_set_session_id_context(ssl,
                                        (unsigned char *)&id,
                                        sizeof(id));
 
-            /* Toggle the renegotiation state to allow the new
-             * handshake to proceed. */
-            sslconn->reneg_state = RENEG_ALLOW;
-            
             SSL_renegotiate(ssl);
             SSL_do_handshake(ssl);
 
             if (SSL_get_state(ssl) != SSL_ST_OK) {
                 ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
                              "Re-negotiation request failed");
-                ssl_log_ssl_error(APLOG_MARK, APLOG_ERR, r->server);
 
                 r->connection->aborted = 1;
                 return HTTP_FORBIDDEN;
@@ -677,11 +632,13 @@ int ssl_hook_Access(request_rec *r)
             ap_log_error(APLOG_MARK, APLOG_INFO, 0, r->server,
                          "Awaiting re-negotiation handshake");
 
+            /* XXX: Should replace SSL_set_state with SSL_renegotiate(ssl);
+             * However, this causes failures in perl-framework currently,
+             * perhaps pre-test if we have already negotiated?
+             */
             SSL_set_state(ssl, SSL_ST_ACCEPT);
             SSL_do_handshake(ssl);
 
-            sslconn->reneg_state = RENEG_REJECT;
-
             if (SSL_get_state(ssl) != SSL_ST_OK) {
                 ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
                              "Re-negotiation handshake failed: "
@@ -729,7 +686,7 @@ int ssl_hook_Access(request_rec *r)
                 X509_free(peercert);
             }
         }
-        
+
         /*
          * Also check that SSLCipherSuite has been enforced as expected.
          */
@@ -758,7 +715,7 @@ int ssl_hook_Access(request_rec *r)
                                    r, (char *)dc->szUserName);
         if (val && val[0])
             r->user = val;
-    } 
+    }
 
     /*
      * Check SSLRequire boolean expressions
@@ -776,7 +733,7 @@ int ssl_hook_Access(request_rec *r)
                               "SSL requirement expression: %s",
                               ssl_expr_get_error());
 
-            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, 
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
                           "access to %s failed, reason: %s",
                           r->filename, cp);
 
@@ -795,7 +752,7 @@ int ssl_hook_Access(request_rec *r)
             ap_log_error(APLOG_MARK, APLOG_INFO, 0, r->server,
                          "Failed expression: %s", req->cpExpr);
 
-            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, 
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
                           "access to %s failed, reason: %s",
                           r->filename,
                           "SSL requirement expression not fulfilled "
@@ -884,12 +841,13 @@ int ssl_hook_UserCheck(request_rec *r)
      * - ssl not enabled
      * - client did not present a certificate
      */
-    if (!(sc->enabled && sslconn && sslconn->ssl && sslconn->client_cert) ||
+    if (!((sc->enabled == SSL_ENABLED_TRUE || sc->enabled == SSL_ENABLED_OPTIONAL)
+          && sslconn && sslconn->ssl && sslconn->client_cert) ||
         !(dc->nOptions & SSL_OPT_FAKEBASICAUTH) || r->user)
     {
         return DECLINED;
     }
-    
+
     if (!sslconn->client_dn) {
         X509_NAME *name = X509_get_subject_name(sslconn->client_cert);
         char *cp = X509_NAME_oneline(name, NULL, 0);
@@ -910,9 +868,9 @@ int ssl_hook_UserCheck(request_rec *r)
      * adding the string "xxj31ZMTZzkVA" as the password in the user file.
      * This is just the crypted variant of the word "password" ;-)
      */
-    auth_line = apr_pstrcat(r->pool, "Basic ", 
-                            ap_pbase64encode(r->pool, 
-                                             apr_pstrcat(r->pool, clientdn, 
+    auth_line = apr_pstrcat(r->pool, "Basic ",
+                            ap_pbase64encode(r->pool,
+                                             apr_pstrcat(r->pool, clientdn,
                                                          ":password", NULL)),
                             NULL);
     apr_table_set(r->headers_in, "Authorization", auth_line);
@@ -950,7 +908,7 @@ static const char *ssl_hook_Fixup_vars[] = {
     "SSL_VERSION_INTERFACE",
     "SSL_VERSION_LIBRARY",
     "SSL_PROTOCOL",
-    "SSL_SECURE_RENEG",
+    "SSL_COMPRESS_METHOD",
     "SSL_CIPHER",
     "SSL_CIPHER_EXPORT",
     "SSL_CIPHER_USEKEYSIZE",
@@ -960,6 +918,7 @@ static const char *ssl_hook_Fixup_vars[] = {
     "SSL_CLIENT_M_SERIAL",
     "SSL_CLIENT_V_START",
     "SSL_CLIENT_V_END",
+    "SSL_CLIENT_V_REMAIN",
     "SSL_CLIENT_S_DN",
     "SSL_CLIENT_S_DN_C",
     "SSL_CLIENT_S_DN_ST",
@@ -1039,10 +998,14 @@ int ssl_hook_Fixup(request_rec *r)
     SSL *ssl;
     int i;
 
+    if (sc->enabled == SSL_ENABLED_OPTIONAL && !(sslconn && sslconn->ssl)) {
+        apr_table_setn(r->headers_out, "Upgrade", "TLS/1.0, HTTP/1.1");
+    }
+
     /*
      * Check to see if SSL is on
      */
-    if (!(sc->enabled && sslconn && (ssl = sslconn->ssl))) {
+    if (!(((sc->enabled == SSL_ENABLED_TRUE) || (sc->enabled == SSL_ENABLED_OPTIONAL)) && sslconn && (ssl = sslconn->ssl))) {
         return DECLINED;
     }
 
@@ -1050,7 +1013,7 @@ int ssl_hook_Fixup(request_rec *r)
      * Annotate the SSI/CGI environment with standard SSL information
      */
     /* the always present HTTPS (=HTTP over SSL) flag! */
-    apr_table_setn(env, "HTTPS", "on"); 
+    apr_table_setn(env, "HTTPS", "on");
 
     /* standard SSL environment variables */
     if (dc->nOptions & SSL_OPT_STDENVVARS) {
@@ -1089,12 +1052,6 @@ int ssl_hook_Fixup(request_rec *r)
         }
     }
 
-
-#ifdef SSL_get_secure_renegotiation_support
-    apr_table_setn(r->notes, "ssl-secure-reneg", 
-                   SSL_get_secure_renegotiation_support(ssl) ? "1" : "0");
-#endif
-
     return DECLINED;
 }
 
@@ -1148,8 +1105,8 @@ RSA *ssl_callback_TmpRSA(SSL *ssl, int export, int keylen)
     SSLModConfigRec *mc = myModConfig(c->base_server);
     int idx;
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, c->base_server,
-                 "handing out temporary %d bit RSA key", keylen);
+    ap_log_cerror(APLOG_MARK, APLOG_DEBUG, 0, c,
+                  "handing out temporary %d bit RSA key", keylen);
 
     /* doesn't matter if export flag is on,
      * we won't be asked for keylen > 512 in that case.
@@ -1171,7 +1128,7 @@ RSA *ssl_callback_TmpRSA(SSL *ssl, int export, int keylen)
     return (RSA *)mc->pTmpKeys[idx];
 }
 
-/* 
+/*
  * Hand out the already generated DH parameters...
  */
 DH *ssl_callback_TmpDH(SSL *ssl, int export, int keylen)
@@ -1180,8 +1137,8 @@ DH *ssl_callback_TmpDH(SSL *ssl, int export, int keylen)
     SSLModConfigRec *mc = myModConfig(c->base_server);
     int idx;
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, c->base_server,
-                 "handing out temporary %d bit DH key", keylen);
+    ap_log_cerror(APLOG_MARK, APLOG_DEBUG, 0, c,
+                  "handing out temporary %d bit DH key", keylen);
 
     switch (keylen) {
       case 512:
@@ -1254,7 +1211,7 @@ int ssl_callback_SSLVerify(int ok, X509_STORE_CTX *ctx)
     }
 
     if (verify == SSL_CVERIFY_NONE) {
-        /* 
+        /*
          * SSLProxyVerify is either not configured or set to "none".
          * (this callback doesn't happen in the server context if SSLVerify
          *  is not configured or set to "none")
@@ -1419,7 +1376,7 @@ int ssl_callback_SSLVerify_CRL(int ok, X509_STORE_CTX *ctx, conn_rec *c)
 
             BIO_free(bio);
 
-            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, buff);
+            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, "%s", buff);
         }
 
         /*
@@ -1537,7 +1494,7 @@ static void modssl_proxy_info_log(server_rec *s,
     dn = X509_NAME_oneline(name, name_buf, sizeof(name_buf));
 
     ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                 SSLPROXY_CERT_CB_LOG_FMT "%s, sending %s", 
+                 SSLPROXY_CERT_CB_LOG_FMT "%s, sending %s",
                  sc->vhost_id, msg, dn ? dn : "-uknown-");
 }
 
@@ -1552,7 +1509,7 @@ static void modssl_proxy_info_log(server_rec *s,
     *pkey = info->x_pkey->dec_pkey; \
     EVP_PKEY_reference_inc(*pkey)
 
-int ssl_callback_proxy_cert(SSL *ssl, MODSSL_CLIENT_CERT_CB_ARG_TYPE **x509, EVP_PKEY **pkey) 
+int ssl_callback_proxy_cert(SSL *ssl, MODSSL_CLIENT_CERT_CB_ARG_TYPE **x509, EVP_PKEY **pkey)
 {
     conn_rec *c = (conn_rec *)SSL_get_app_data(ssl);
     server_rec *s = c->base_server;
@@ -1562,8 +1519,8 @@ int ssl_callback_proxy_cert(SSL *ssl, MODSSL_CLIENT_CERT_CB_ARG_TYPE **x509, EVP
     STACK_OF(X509_NAME) *ca_list;
     STACK_OF(X509_INFO) *certs = sc->proxy->pkp->certs;
     int i, j;
-    
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, 
+
+    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
                  SSLPROXY_CERT_CB_LOG_FMT "entered",
                  sc->vhost_id);
 
@@ -1573,23 +1530,23 @@ int ssl_callback_proxy_cert(SSL *ssl, MODSSL_CLIENT_CERT_CB_ARG_TYPE **x509, EVP
                      "downstream server wanted client certificate "
                      "but none are configured", sc->vhost_id);
         return FALSE;
-    }                                                                     
+    }
 
     ca_list = SSL_get_client_CA_list(ssl);
 
     if (!ca_list || (sk_X509_NAME_num(ca_list) <= 0)) {
-        /* 
-         * downstream server didn't send us a list of acceptable CA certs, 
+        /*
+         * downstream server didn't send us a list of acceptable CA certs,
          * so we send the first client cert in the list.
-         */   
+         */
         info = sk_X509_INFO_value(certs, 0);
-        
+
         modssl_proxy_info_log(s, info, "no acceptable CA list");
 
         modssl_set_cert_info(info, x509, pkey);
 
         return TRUE;
-    }         
+    }
 
     for (i = 0; i < sk_X509_NAME_num(ca_list); i++) {
         ca_name = sk_X509_NAME_value(ca_list, i);
@@ -1612,7 +1569,7 @@ int ssl_callback_proxy_cert(SSL *ssl, MODSSL_CLIENT_CERT_CB_ARG_TYPE **x509, EVP
                  SSLPROXY_CERT_CB_LOG_FMT
                  "no client certificate found!?", sc->vhost_id);
 
-    return FALSE; 
+    return FALSE;
 }
 
 static void ssl_session_log(server_rec *s,
@@ -1761,55 +1718,76 @@ void ssl_callback_DelSessionCacheEntry(SSL_CTX *ctx,
     return;
 }
 
-/* Dump debugginfo trace to the log file. */
-static void log_tracing_state(MODSSL_INFO_CB_ARG_TYPE ssl, conn_rec *c,
-                              server_rec *s, int where, int rc)
+/*
+ * This callback function is executed while OpenSSL processes the
+ * SSL handshake and does SSL record layer stuff. We use it to
+ * trace OpenSSL's processing in out SSL logfile.
+ */
+void ssl_callback_LogTracingState(MODSSL_INFO_CB_ARG_TYPE ssl, int where, int rc)
 {
+    conn_rec *c;
+    server_rec *s;
+    SSLSrvConfigRec *sc;
+
+    /*
+     * find corresponding server
+     */
+    if (!(c = (conn_rec *)SSL_get_app_data((SSL *)ssl))) {
+        return;
+    }
+
+    s = c->base_server;
+    if (!(sc = mySrvConfig(s))) {
+        return;
+    }
+
     /*
      * create the various trace messages
      */
-    if (where & SSL_CB_HANDSHAKE_START) {
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                     "%s: Handshake: start", SSL_LIBRARY_NAME);
-    }
-    else if (where & SSL_CB_HANDSHAKE_DONE) {
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                     "%s: Handshake: done", SSL_LIBRARY_NAME);
-    }
-    else if (where & SSL_CB_LOOP) {
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                     "%s: Loop: %s",
-                     SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
-    }
-    else if (where & SSL_CB_READ) {
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                     "%s: Read: %s",
-                     SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
-    }
-    else if (where & SSL_CB_WRITE) {
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                     "%s: Write: %s",
-                     SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
-    }
-    else if (where & SSL_CB_ALERT) {
-        char *str = (where & SSL_CB_READ) ? "read" : "write";
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                     "%s: Alert: %s:%s:%s",
-                     SSL_LIBRARY_NAME, str,
-                     SSL_alert_type_string_long(rc),
-                     SSL_alert_desc_string_long(rc));
-    }
-    else if (where & SSL_CB_EXIT) {
-        if (rc == 0) {
+    if (s->loglevel >= APLOG_DEBUG) {
+        if (where & SSL_CB_HANDSHAKE_START) {
             ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                         "%s: Exit: failed in %s",
+                         "%s: Handshake: start", SSL_LIBRARY_NAME);
+        }
+        else if (where & SSL_CB_HANDSHAKE_DONE) {
+            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
+                         "%s: Handshake: done", SSL_LIBRARY_NAME);
+        }
+        else if (where & SSL_CB_LOOP) {
+            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
+                         "%s: Loop: %s",
                          SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
         }
-        else if (rc < 0) {
+        else if (where & SSL_CB_READ) {
             ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                         "%s: Exit: error in %s",
+                         "%s: Read: %s",
                          SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
         }
+        else if (where & SSL_CB_WRITE) {
+            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
+                         "%s: Write: %s",
+                         SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
+        }
+        else if (where & SSL_CB_ALERT) {
+            char *str = (where & SSL_CB_READ) ? "read" : "write";
+            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
+                         "%s: Alert: %s:%s:%s",
+                         SSL_LIBRARY_NAME, str,
+                         SSL_alert_type_string_long(rc),
+                         SSL_alert_desc_string_long(rc));
+        }
+        else if (where & SSL_CB_EXIT) {
+            if (rc == 0) {
+                ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
+                             "%s: Exit: failed in %s",
+                             SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
+            }
+            else if (rc < 0) {
+                ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
+                             "%s: Exit: error in %s",
+                             SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
+            }
+        }
     }
 
     /*
@@ -1829,48 +1807,3 @@ static void log_tracing_state(MODSSL_INFO_CB_ARG_TYPE ssl, conn_rec *c,
     }
 }
 
-/*
- * This callback function is executed while OpenSSL processes the SSL
- * handshake and does SSL record layer stuff.  It's used to trap
- * client-initiated renegotiations, and for dumping everything to the
- * log.
- */
-void ssl_callback_Info(MODSSL_INFO_CB_ARG_TYPE ssl, int where, int rc)
-{
-    conn_rec *c;
-    server_rec *s;
-    SSLConnRec *scr;
-
-    /* Retrieve the conn_rec and the associated SSLConnRec. */
-    if ((c = (conn_rec *)SSL_get_app_data((SSL *)ssl)) == NULL) {
-        return;
-    }
-
-    if ((scr = myConnConfig(c)) == NULL) {
-        return;
-    }
-
-    /* If the reneg state is to reject renegotiations, check the SSL
-     * state machine and move to ABORT if a Client Hello is being
-     * read. */
-    if ((where & SSL_CB_ACCEPT_LOOP) && scr->reneg_state == RENEG_REJECT) {
-        int state = SSL_get_state((SSL *)ssl);
-        
-        if (state == SSL3_ST_SR_CLNT_HELLO_A 
-            || state == SSL23_ST_SR_CLNT_HELLO_A) {
-            scr->reneg_state = RENEG_ABORT;
-            ap_log_cerror(APLOG_MARK, APLOG_ERR, 0, c,
-                          "rejecting client initiated renegotiation");
-        }
-    }
-    /* If the first handshake is complete, change state to reject any
-     * subsequent client-initated renegotiation. */
-    else if ((where & SSL_CB_HANDSHAKE_DONE) && scr->reneg_state == RENEG_INIT) {
-        scr->reneg_state = RENEG_REJECT;
-    }
-
-    s = c->base_server;
-    if (s && s->loglevel >= APLOG_DEBUG) {
-        log_tracing_state(ssl, c, s, where, rc);
-    }
-}
