@@ -1,9 +1,9 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 2001-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -24,11 +24,11 @@
  *  Session Cache via Shared Memory (Cyclic Buffer Variant)
  */
 
-#include "mod_ssl.h"
+#include "ssl_private.h"
 
-/* 
+/*
  * This shared memory based SSL session cache implementation was
- * originally written by Geoff Thorpe <geoff@geoffthorpe.net> for C2Net
+ * originally written by Geoff Thorpe <geoff geoffthorpe.net> for C2Net
  * Europe as a contribution to Ralf Engelschall's mod_ssl project.
  */
 
@@ -88,7 +88,7 @@
  *   how many session-retrieves have failed.
  * - removes_hit (unsigned long):
  * - removes_miss (unsigned long):
- * 
+ *
  * Following immediately after the header is an array of "divisions".
  * Each division is simply a "queue" immediately followed by its
  * corresponding "cache". Each division handles some pre-defined band
@@ -135,7 +135,7 @@
  * sessions are stored.
  */
 
-/* 
+/*
  * Header - can be memcpy'd to and from the front of the shared
  * memory segment. NB: The first copy (commented out) has the
  * elements in a meaningful order, but due to data-alignment
@@ -161,7 +161,7 @@ typedef struct {
     unsigned int index_size;
 } SHMCBHeader;
 
-/* 
+/*
  * Index - can be memcpy'd to and from an index inside each
  * queue's index array.
  */
@@ -172,7 +172,7 @@ typedef struct {
     unsigned char removed;
 } SHMCBIndex;
 
-/* 
+/*
  * Queue - must be populated by a call to shmcb_get_division
  * and the structure's pointers are used for updating (ie.
  * the structure doesn't need any "set" to update values).
@@ -184,7 +184,7 @@ typedef struct {
     SHMCBIndex *indexes;
 } SHMCBQueue;
 
-/* 
+/*
  * Cache - same comment as for Queue. 'Queue's are in a 1-1
  * correspondance with 'Cache's and are usually carried round
  * in a pair, they are only seperated for clarity.
@@ -210,35 +210,35 @@ typedef struct {
 static unsigned int shmcb_get_safe_uint(unsigned int *);
 static void shmcb_set_safe_uint_ex(unsigned char *, const unsigned char *);
 #define shmcb_set_safe_uint(pdest, src) \
-	do { \
-		unsigned int tmp_uint = src; \
-		shmcb_set_safe_uint_ex((unsigned char *)pdest, \
-			(const unsigned char *)(&tmp_uint)); \
-	} while(0)
+        do { \
+                unsigned int tmp_uint = src; \
+                shmcb_set_safe_uint_ex((unsigned char *)pdest, \
+                        (const unsigned char *)(&tmp_uint)); \
+        } while(0)
 #if 0 /* Unused so far */
 static unsigned long shmcb_get_safe_ulong(unsigned long *);
 static void shmcb_set_safe_ulong_ex(unsigned char *, const unsigned char *);
 #define shmcb_set_safe_ulong(pdest, src) \
-	do { \
-		unsigned long tmp_ulong = src; \
-		shmcb_set_safe_ulong_ex((unsigned char *)pdest, \
-			(const unsigned char *)(&tmp_ulong)); \
-	} while(0)
+        do { \
+                unsigned long tmp_ulong = src; \
+                shmcb_set_safe_ulong_ex((unsigned char *)pdest, \
+                        (const unsigned char *)(&tmp_ulong)); \
+        } while(0)
 #endif
 static time_t shmcb_get_safe_time(time_t *);
 static void shmcb_set_safe_time_ex(unsigned char *, const unsigned char *);
 #define shmcb_set_safe_time(pdest, src) \
-	do { \
-		time_t tmp_time = src; \
-		shmcb_set_safe_time_ex((unsigned char *)pdest, \
-			(const unsigned char *)(&tmp_time)); \
-	} while(0)
+        do { \
+                time_t tmp_time = src; \
+                shmcb_set_safe_time_ex((unsigned char *)pdest, \
+                        (const unsigned char *)(&tmp_time)); \
+        } while(0)
 
 /* This is necessary simply so that the size passed to memset() is not a
  * compile-time constant, preventing the compiler from optimising it. */
 static void shmcb_safe_clear(void *ptr, size_t size)
 {
-	memset(ptr, 0, size);
+        memset(ptr, 0, size);
 }
 
 /* Underlying functions for session-caching */
@@ -279,12 +279,12 @@ static unsigned int shmcb_get_safe_uint(unsigned int *ptr)
 {
     unsigned int ret;
     shmcb_set_safe_uint_ex((unsigned char *)(&ret),
-		    (const unsigned char *)ptr);
+                    (const unsigned char *)ptr);
     return ret;
 }
 
 static void shmcb_set_safe_uint_ex(unsigned char *dest,
-				const unsigned char *src)
+                                const unsigned char *src)
 {
     memcpy(dest, src, sizeof(unsigned int));
 }
@@ -294,12 +294,12 @@ static unsigned long shmcb_get_safe_ulong(unsigned long *ptr)
 {
     unsigned long ret;
     shmcb_set_safe_ulong_ex((unsigned char *)(&ret),
-		    (const unsigned char *)ptr);
+                    (const unsigned char *)ptr);
     return ret;
 }
 
 static void shmcb_set_safe_ulong_ex(unsigned char *dest,
-				const unsigned char *src)
+                                const unsigned char *src)
 {
     memcpy(dest, src, sizeof(unsigned long));
 }
@@ -309,12 +309,12 @@ static time_t shmcb_get_safe_time(time_t * ptr)
 {
     time_t ret;
     shmcb_set_safe_time_ex((unsigned char *)(&ret),
-		    (const unsigned char *)ptr);
+                    (const unsigned char *)ptr);
     return ret;
 }
 
 static void shmcb_set_safe_time_ex(unsigned char *dest,
-				const unsigned char *src)
+                                const unsigned char *src)
 {
     memcpy(dest, src, sizeof(time_t));
 }
@@ -341,13 +341,17 @@ void ssl_scache_shmcb_init(server_rec *s, apr_pool_t *p)
     }
 
     /* Use anonymous shm by default, fall back on name-based. */
-    rv = apr_shm_create(&(mc->pSessionCacheDataMM), 
-                        mc->nSessionCacheDataSize, 
+    rv = apr_shm_create(&(mc->pSessionCacheDataMM),
+                        mc->nSessionCacheDataSize,
                         NULL, mc->pPool);
-    
+
     if (APR_STATUS_IS_ENOTIMPL(rv)) {
-        rv = apr_shm_create(&(mc->pSessionCacheDataMM), 
-                            mc->nSessionCacheDataSize, 
+        /* For a name-based segment, remove it first in case of a
+         * previous unclean shutdown. */
+        apr_shm_remove(mc->szSessionCacheDataFile, mc->pPool);
+
+        rv = apr_shm_create(&(mc->pSessionCacheDataMM),
+                            mc->nSessionCacheDataSize,
                             mc->szSessionCacheDataFile,
                             mc->pPool);
     }
@@ -363,7 +367,7 @@ void ssl_scache_shmcb_init(server_rec *s, apr_pool_t *p)
     shm_segsize = apr_shm_size_get(mc->pSessionCacheDataMM);
 
     ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                 "shmcb_init allocated %" APR_SIZE_T_FMT 
+                 "shmcb_init allocated %" APR_SIZE_T_FMT
                  " bytes of shared memory",
                  shm_segsize);
     if (!shmcb_init_memory(s, shm_segment, shm_segsize)) {
@@ -374,12 +378,10 @@ void ssl_scache_shmcb_init(server_rec *s, apr_pool_t *p)
     ap_log_error(APLOG_MARK, APLOG_INFO, 0, s,
                  "Shared memory session cache initialised");
 
-    /* 
-     * Success ... we hack the memory block into place by cheating for
-     * now and stealing a member variable the original shared memory
-     * cache was using. :-)
+    /*
+     * Success ...
      */
-    mc->tSessionCacheDataTable = (table_t *) shm_segment;
+    mc->tSessionCacheDataTable = shm_segment;
     return;
 }
 
@@ -398,13 +400,11 @@ BOOL ssl_scache_shmcb_store(server_rec *s, UCHAR *id, int idlen,
                            time_t timeout, SSL_SESSION * pSession)
 {
     SSLModConfigRec *mc = myModConfig(s);
-    void *shm_segment;
     BOOL to_return = FALSE;
 
-    /* We've kludged our pointer into the other cache's member variable. */
-    shm_segment = (void *) mc->tSessionCacheDataTable;
     ssl_mutex_on(s);
-    if (!shmcb_store_session(s, shm_segment, id, idlen, pSession, timeout))
+    if (!shmcb_store_session(s, mc->tSessionCacheDataTable, id, idlen,
+                             pSession, timeout))
         /* in this cache engine, "stores" should never fail. */
         ap_log_error(APLOG_MARK, APLOG_ERR, 0, s,
                      "'shmcb' code was unable to store a "
@@ -421,13 +421,10 @@ BOOL ssl_scache_shmcb_store(server_rec *s, UCHAR *id, int idlen,
 SSL_SESSION *ssl_scache_shmcb_retrieve(server_rec *s, UCHAR *id, int idlen)
 {
     SSLModConfigRec *mc = myModConfig(s);
-    void *shm_segment;
     SSL_SESSION *pSession;
 
-    /* We've kludged our pointer into the other cache's member variable. */
-    shm_segment = (void *) mc->tSessionCacheDataTable;
     ssl_mutex_on(s);
-    pSession = shmcb_retrieve_session(s, shm_segment, id, idlen);
+    pSession = shmcb_retrieve_session(s, mc->tSessionCacheDataTable, id, idlen);
     ssl_mutex_off(s);
     if (pSession)
         ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
@@ -445,43 +442,28 @@ SSL_SESSION *ssl_scache_shmcb_retrieve(server_rec *s, UCHAR *id, int idlen)
 void ssl_scache_shmcb_remove(server_rec *s, UCHAR *id, int idlen)
 {
     SSLModConfigRec *mc = myModConfig(s);
-    void *shm_segment;
 
-    /* We've kludged our pointer into the other cache's member variable. */
-    shm_segment = (void *) mc->tSessionCacheDataTable;
     ssl_mutex_on(s);
-    shmcb_remove_session(s, shm_segment, id, idlen);
+    shmcb_remove_session(s, mc->tSessionCacheDataTable, id, idlen);
     ssl_mutex_off(s);
 }
 
-void ssl_scache_shmcb_expire(server_rec *s)
+void ssl_scache_shmcb_status(request_rec *r, int flags, apr_pool_t *p)
 {
-    /* NOP */
-    return;
-}
-
-void ssl_scache_shmcb_status(server_rec *s, apr_pool_t *p,
-                            void (*func) (char *, void *), void *arg)
-{
-    SSLModConfigRec *mc = myModConfig(s);
+    SSLModConfigRec *mc = myModConfig(r->server);
     SHMCBHeader *header;
     SHMCBQueue queue;
     SHMCBCache cache;
     SHMCBIndex *idx;
-    void *shm_segment;
     unsigned int loop, total, cache_total, non_empty_divisions;
     int index_pct, cache_pct;
     double expiry_total;
     time_t average_expiry, now, max_expiry, min_expiry, idxexpiry;
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, 
-                 "inside ssl_scache_shmcb_status");
-
-    /* We've kludged our pointer into the other cache's member variable. */
-    shm_segment = (void *) mc->tSessionCacheDataTable;
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "inside shmcb_status");
 
     /* Get the header structure. */
-    shmcb_get_header(shm_segment, &header);
+    shmcb_get_header(mc->tSessionCacheDataTable, &header);
     total = cache_total = non_empty_divisions = 0;
     average_expiry = max_expiry = min_expiry = 0;
     expiry_total = 0;
@@ -492,7 +474,7 @@ void ssl_scache_shmcb_status(server_rec *s, apr_pool_t *p,
     now = time(NULL);
     for (loop = 0; loop <= header->division_mask; loop++) {
         if (shmcb_get_division(header, &queue, &cache, loop)) {
-            shmcb_expire_division(s, &queue, &cache);
+            shmcb_expire_division(r->server, &queue, &cache);
             total += shmcb_get_safe_uint(queue.pos_count);
             cache_total += shmcb_get_safe_uint(cache.pos_count);
             if (shmcb_get_safe_uint(queue.pos_count) > 0) {
@@ -513,46 +495,45 @@ void ssl_scache_shmcb_status(server_rec *s, apr_pool_t *p,
     }
     index_pct = (100 * total) / (header->index_num * (header->division_mask + 1));
     cache_pct = (100 * cache_total) / (header->cache_data_size * (header->division_mask + 1));
-    func(apr_psprintf(p, "cache type: <b>SHMCB</b>, shared memory: <b>%d</b> "
-                     "bytes, current sessions: <b>%d</b><br>",
-                     mc->nSessionCacheDataSize, total), arg);
-    func(apr_psprintf(p, "sub-caches: <b>%d</b>, indexes per sub-cache: "
-                     "<b>%d</b><br>", (int) header->division_mask + 1,
-                     (int) header->index_num), arg);
+    ap_rprintf(r, "cache type: <b>SHMCB</b>, shared memory: <b>%d</b> "
+               "bytes, current sessions: <b>%d</b><br>",
+               mc->nSessionCacheDataSize, total);
+    ap_rprintf(r, "sub-caches: <b>%d</b>, indexes per sub-cache: "
+               "<b>%d</b><br>", (int) header->division_mask + 1,
+               (int) header->index_num);
     if (non_empty_divisions != 0) {
         average_expiry = (time_t)(expiry_total / (double)non_empty_divisions);
-        func(apr_psprintf(p, "time left on oldest entries' SSL sessions: "), arg);
+        ap_rprintf(r, "time left on oldest entries' SSL sessions: ");
         if (now < average_expiry)
-            func(apr_psprintf(p, "avg: <b>%d</b> seconds, (range: %d...%d)<br>",
-                            (int)(average_expiry - now), (int) (min_expiry - now),
-                            (int)(max_expiry - now)), arg);
+            ap_rprintf(r, "avg: <b>%d</b> seconds, (range: %d...%d)<br>",
+                       (int)(average_expiry - now), (int) (min_expiry - now),
+                       (int)(max_expiry - now));
         else
-            func(apr_psprintf(p, "expiry threshold: <b>Calculation Error!</b>" 
-                             "<br>"), arg);
+            ap_rprintf(r, "expiry threshold: <b>Calculation Error!</b>"
+                       "<br>");
 
     }
-    func(apr_psprintf(p, "index usage: <b>%d%%</b>, cache usage: <b>%d%%</b>"
-                     "<br>", index_pct, cache_pct), arg);
-    func(apr_psprintf(p, "total sessions stored since starting: <b>%lu</b><br>",
-                     header->num_stores), arg);
-    func(apr_psprintf(p,"total sessions expired since starting: <b>%lu</b><br>",
-                     header->num_expiries), arg);
-    func(apr_psprintf(p, "total (pre-expiry) sessions scrolled out of the "
-                     "cache: <b>%lu</b><br>", header->num_scrolled), arg);
-    func(apr_psprintf(p, "total retrieves since starting: <b>%lu</b> hit, "
-                     "<b>%lu</b> miss<br>", header->num_retrieves_hit,
-                     header->num_retrieves_miss), arg);
-    func(apr_psprintf(p, "total removes since starting: <b>%lu</b> hit, "
-                     "<b>%lu</b> miss<br>", header->num_removes_hit,
-                     header->num_removes_miss), arg);
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, 
-                 "leaving shmcb_status");
+    ap_rprintf(r, "index usage: <b>%d%%</b>, cache usage: <b>%d%%</b>"
+               "<br>", index_pct, cache_pct);
+    ap_rprintf(r, "total sessions stored since starting: <b>%lu</b><br>",
+               header->num_stores);
+    ap_rprintf(r, "total sessions expired since starting: <b>%lu</b><br>",
+               header->num_expiries);
+    ap_rprintf(r, "total (pre-expiry) sessions scrolled out of the "
+               "cache: <b>%lu</b><br>", header->num_scrolled);
+    ap_rprintf(r, "total retrieves since starting: <b>%lu</b> hit, "
+               "<b>%lu</b> miss<br>", header->num_retrieves_hit,
+               header->num_retrieves_miss);
+    ap_rprintf(r, "total removes since starting: <b>%lu</b> hit, "
+               "<b>%lu</b> miss<br>", header->num_removes_hit,
+               header->num_removes_miss);
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "leaving shmcb_status");
     return;
 }
 
 /*
 **
-** Memory manipulation and low-level cache operations 
+** Memory manipulation and low-level cache operations
 **
 */
 
@@ -565,7 +546,7 @@ static BOOL shmcb_init_memory(
     SHMCBCache cache;
     unsigned int temp, loop, granularity;
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s, 
+    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
                  "entered shmcb_init_memory()");
 
     /* Calculate some sizes... */
@@ -803,7 +784,7 @@ static BOOL shmcb_remove_session(
 }
 
 
-/* 
+/*
 **
 ** Weirdo cyclic buffer functions
 **
@@ -1034,7 +1015,7 @@ static unsigned int shmcb_expire_division(
             shmcb_set_safe_uint(queue->first_pos, new_pos);
             /* peek to the start of the next session */
             idx = shmcb_get_index(queue, new_pos);
-            /* We can use shmcb_cyclic_space because we've guaranteed 
+            /* We can use shmcb_cyclic_space because we've guaranteed
              * we don't fit the ambiguous full/empty case. */
             shmcb_set_safe_uint(cache->pos_count,
                                shmcb_get_safe_uint(cache->pos_count) -
@@ -1333,7 +1314,7 @@ static BOOL shmcb_remove_session_id(
             session_id_length = SSL_SESSION_get_session_id_length(pSession);
             session_id = SSL_SESSION_get_session_id(pSession);
 
-            if ((session_id_length == idlen) 
+            if ((session_id_length == idlen)
                  && (memcmp(id, session_id, idlen) == 0)) {
                 ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
                             "a match!");
