@@ -1,9 +1,9 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 2002-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -29,7 +29,7 @@ struct ap_regkey_t {
 
 AP_DECLARE(const ap_regkey_t *) ap_regkey_const(int i)
 {
-    static struct ap_regkey_t ap_regkey_consts[7] = 
+    static struct ap_regkey_t ap_regkey_consts[7] =
     {
         {NULL, HKEY_CLASSES_ROOT},
         {NULL, HKEY_CURRENT_CONFIG},
@@ -55,10 +55,10 @@ apr_status_t regkey_cleanup(void *key)
 }
 
 
-AP_DECLARE(apr_status_t) ap_regkey_open(ap_regkey_t **newkey, 
+AP_DECLARE(apr_status_t) ap_regkey_open(ap_regkey_t **newkey,
                                         const ap_regkey_t *parentkey,
                                         const char *keyname,
-                                        apr_int32_t flags, 
+                                        apr_int32_t flags,
                                         apr_pool_t *pool)
 {
     DWORD access = KEY_QUERY_VALUE;
@@ -69,10 +69,10 @@ AP_DECLARE(apr_status_t) ap_regkey_open(ap_regkey_t **newkey,
     if (flags & APR_READ)
         access |= KEY_READ;
     if (flags & APR_WRITE)
-        access |= KEY_WRITE; 
+        access |= KEY_WRITE;
 
 #if APR_HAS_UNICODE_FS
-    IF_WIN_OS_IS_UNICODE 
+    IF_WIN_OS_IS_UNICODE
     {
         apr_size_t keylen = strlen(keyname) + 1;
         apr_size_t wkeylen = 256;
@@ -84,7 +84,7 @@ AP_DECLARE(apr_status_t) ap_regkey_open(ap_regkey_t **newkey,
             return APR_ENAMETOOLONG;
 
         if (flags & APR_CREATE)
-            rc = RegCreateKeyExW(parentkey->hkey, wkeyname, 0, NULL, 0, 
+            rc = RegCreateKeyExW(parentkey->hkey, wkeyname, 0, NULL, 0,
                                  access, NULL, &hkey, &exists);
         else
             rc = RegOpenKeyExW(parentkey->hkey, wkeyname, 0, access, &hkey);
@@ -94,7 +94,7 @@ AP_DECLARE(apr_status_t) ap_regkey_open(ap_regkey_t **newkey,
     ELSE_WIN_OS_IS_ANSI
     {
         if (flags & APR_CREATE)
-            rc = RegCreateKeyEx(parentkey->hkey, keyname, 0, NULL, 0, 
+            rc = RegCreateKeyEx(parentkey->hkey, keyname, 0, NULL, 0,
                                 access, NULL, &hkey, &exists);
         else
             rc = RegOpenKeyEx(parentkey->hkey, keyname, 0, access, &hkey);
@@ -111,7 +111,7 @@ AP_DECLARE(apr_status_t) ap_regkey_open(ap_regkey_t **newkey,
     *newkey = apr_palloc(pool, sizeof(**newkey));
     (*newkey)->pool = pool;
     (*newkey)->hkey = hkey;
-    apr_pool_cleanup_register((*newkey)->pool, (void *)(*newkey), 
+    apr_pool_cleanup_register((*newkey)->pool, (void *)(*newkey),
                               regkey_cleanup, apr_pool_cleanup_null);
     return APR_SUCCESS;
 }
@@ -127,14 +127,14 @@ AP_DECLARE(apr_status_t) ap_regkey_close(ap_regkey_t *regkey)
 }
 
 
-AP_DECLARE(apr_status_t) ap_regkey_remove(const ap_regkey_t *parent, 
+AP_DECLARE(apr_status_t) ap_regkey_remove(const ap_regkey_t *parent,
                                           const char *keyname,
                                           apr_pool_t *pool)
 {
     LONG rc;
 
 #if APR_HAS_UNICODE_FS
-    IF_WIN_OS_IS_UNICODE 
+    IF_WIN_OS_IS_UNICODE
     {
         apr_size_t keylen = strlen(keyname) + 1;
         apr_size_t wkeylen = 256;
@@ -176,9 +176,9 @@ AP_DECLARE(apr_status_t) ap_regkey_remove(const ap_regkey_t *parent,
 }
 
 
-AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result, 
-                                             ap_regkey_t *key, 
-                                             const char *valuename, 
+AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result,
+                                             ap_regkey_t *key,
+                                             const char *valuename,
                                              apr_pool_t *pool)
 {
     /* Retrieve a registry string value, and explode any envvars
@@ -186,10 +186,10 @@ AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result,
      */
     LONG rc;
     DWORD type;
-    DWORD size = 0;
-    
+    apr_size_t size = 0;
+
 #if APR_HAS_UNICODE_FS
-    IF_WIN_OS_IS_UNICODE 
+    IF_WIN_OS_IS_UNICODE
     {
         apr_size_t valuelen = strlen(valuename) + 1;
         apr_size_t wvallen = 256;
@@ -202,7 +202,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result,
         else if (valuelen)
             return APR_ENAMETOOLONG;
         /* Read to NULL buffer to determine value size */
-        rc = RegQueryValueExW(key->hkey, wvalname, 0, &type, NULL, &size);
+        rc = RegQueryValueExW(key->hkey, wvalname, 0, &type, NULL, (DWORD *)&size);
         if (rc != ERROR_SUCCESS) {
             return APR_FROM_OS_ERROR(rc);
         }
@@ -212,8 +212,8 @@ AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result,
 
         wvalue = apr_palloc(pool, size);
         /* Read value based on size query above */
-        rc = RegQueryValueExW(key->hkey, wvalname, 0, &type, 
-                              (LPBYTE)wvalue, &size);
+        rc = RegQueryValueExW(key->hkey, wvalname, 0, &type,
+                              (LPBYTE)wvalue, (DWORD *)&size);
         if (rc != ERROR_SUCCESS) {
             return APR_FROM_OS_ERROR(rc);
         }
@@ -224,7 +224,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result,
                 apr_wchar_t *tmp = wvalue;
                 /* The size returned by ExpandEnvironmentStringsW is wchars */
                 wvalue = apr_palloc(pool, size * 2);
-                size = ExpandEnvironmentStringsW(tmp, wvalue, size);
+                size = ExpandEnvironmentStringsW(tmp, wvalue, (DWORD)size);
             }
         }
         else {
@@ -248,7 +248,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result,
     ELSE_WIN_OS_IS_ANSI
     {
         /* Read to NULL buffer to determine value size */
-        rc = RegQueryValueEx(key->hkey, valuename, 0, &type, NULL, &size);
+        rc = RegQueryValueEx(key->hkey, valuename, 0, &type, NULL, (DWORD *)&size);
         if (rc != ERROR_SUCCESS)
             return APR_FROM_OS_ERROR(rc);
 
@@ -258,7 +258,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result,
 
         *result = apr_palloc(pool, size);
         /* Read value based on size query above */
-        rc = RegQueryValueEx(key->hkey, valuename, 0, &type, *result, &size);
+        rc = RegQueryValueEx(key->hkey, valuename, 0, &type, *result, (DWORD *)&size);
         if (rc != ERROR_SUCCESS)
             return APR_FROM_OS_ERROR(rc);
 
@@ -271,7 +271,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result,
             if (size) {
                 char *tmp = *result;
                 *result = apr_palloc(pool, size);
-                size = ExpandEnvironmentStrings(tmp, *result, size);
+                size = ExpandEnvironmentStrings(tmp, *result, (DWORD)size);
             }
         }
     }
@@ -280,21 +280,21 @@ AP_DECLARE(apr_status_t) ap_regkey_value_get(char **result,
 }
 
 
-AP_DECLARE(apr_status_t) ap_regkey_value_set(ap_regkey_t *key, 
-                                             const char *valuename, 
-                                             const char *value, 
-                                             apr_int32_t flags, 
+AP_DECLARE(apr_status_t) ap_regkey_value_set(ap_regkey_t *key,
+                                             const char *valuename,
+                                             const char *value,
+                                             apr_int32_t flags,
                                              apr_pool_t *pool)
 {
     /* Retrieve a registry string value, and explode any envvars
      * that the system has configured (e.g. %SystemRoot%/someapp.exe)
      */
     LONG rc;
-    DWORD size = strlen(value) + 1;
+    apr_size_t size = strlen(value) + 1;
     DWORD type = (flags & AP_REGKEY_EXPAND) ? REG_EXPAND_SZ : REG_SZ;
-    
+
 #if APR_HAS_UNICODE_FS
-    IF_WIN_OS_IS_UNICODE 
+    IF_WIN_OS_IS_UNICODE
     {
         apr_size_t alloclen;
         apr_size_t valuelen = strlen(valuename) + 1;
@@ -307,7 +307,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_set(ap_regkey_t *key,
             return rv;
         else if (valuelen)
             return APR_ENAMETOOLONG;
-        
+
         wvallen = alloclen = size;
         wvalue = apr_palloc(pool, alloclen * 2);
         rv = apr_conv_utf8_to_ucs2(value, &size, wvalue, &wvallen);
@@ -320,8 +320,8 @@ AP_DECLARE(apr_status_t) ap_regkey_value_set(ap_regkey_t *key,
          * converted to bytes; the trailing L'\0' continues to be counted.
          */
         size = (alloclen - wvallen) * 2;
-        rc = RegSetValueExW(key->hkey, wvalname, 0, type, 
-                            (LPBYTE)wvalue, size);
+        rc = RegSetValueExW(key->hkey, wvalname, 0, type,
+                            (LPBYTE)wvalue, (DWORD)size);
         if (rc != ERROR_SUCCESS)
             return APR_FROM_OS_ERROR(rc);
     }
@@ -329,7 +329,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_set(ap_regkey_t *key,
 #if APR_HAS_ANSI_FS
     ELSE_WIN_OS_IS_ANSI
     {
-        rc = RegSetValueEx(key->hkey, valuename, 0, type, value, size);
+        rc = RegSetValueEx(key->hkey, valuename, 0, type, value, (DWORD)size);
         if (rc != ERROR_SUCCESS)
             return APR_FROM_OS_ERROR(rc);
     }
@@ -338,20 +338,20 @@ AP_DECLARE(apr_status_t) ap_regkey_value_set(ap_regkey_t *key,
 }
 
 
-AP_DECLARE(apr_status_t) ap_regkey_value_raw_get(void **result, 
+AP_DECLARE(apr_status_t) ap_regkey_value_raw_get(void **result,
                                                  apr_size_t *resultsize,
                                                  apr_int32_t *resulttype,
-                                                 ap_regkey_t *key, 
-                                                 const char *valuename, 
+                                                 ap_regkey_t *key,
+                                                 const char *valuename,
                                                  apr_pool_t *pool)
 {
     /* Retrieve a registry string value, and explode any envvars
      * that the system has configured (e.g. %SystemRoot%/someapp.exe)
      */
     LONG rc;
-    
+
 #if APR_HAS_UNICODE_FS
-    IF_WIN_OS_IS_UNICODE 
+    IF_WIN_OS_IS_UNICODE
     {
         apr_size_t valuelen = strlen(valuename) + 1;
         apr_size_t wvallen = 256;
@@ -363,31 +363,31 @@ AP_DECLARE(apr_status_t) ap_regkey_value_raw_get(void **result,
         else if (valuelen)
             return APR_ENAMETOOLONG;
         /* Read to NULL buffer to determine value size */
-        rc = RegQueryValueExW(key->hkey, wvalname, 0, resulttype, 
-                              NULL, resultsize);
+        rc = RegQueryValueExW(key->hkey, wvalname, 0, resulttype,
+                              NULL, (LPDWORD)resultsize);
         if (rc != ERROR_SUCCESS) {
             return APR_FROM_OS_ERROR(rc);
         }
 
         /* Read value based on size query above */
         *result = apr_palloc(pool, *resultsize);
-        rc = RegQueryValueExW(key->hkey, wvalname, 0, resulttype, 
-                             (LPBYTE)*result, resultsize);
+        rc = RegQueryValueExW(key->hkey, wvalname, 0, resulttype,
+                             (LPBYTE)*result, (LPDWORD)resultsize);
     }
 #endif /* APR_HAS_UNICODE_FS */
 #if APR_HAS_ANSI_FS
     ELSE_WIN_OS_IS_ANSI
     {
         /* Read to NULL buffer to determine value size */
-        rc = RegQueryValueEx(key->hkey, valuename, 0, resulttype, 
-                             NULL, resultsize);
+        rc = RegQueryValueEx(key->hkey, valuename, 0, resulttype,
+                             NULL, (LPDWORD)resultsize);
         if (rc != ERROR_SUCCESS)
             return APR_FROM_OS_ERROR(rc);
 
         /* Read value based on size query above */
         *result = apr_palloc(pool, *resultsize);
-        rc = RegQueryValueEx(key->hkey, valuename, 0, resulttype, 
-                             (LPBYTE)*result, resultsize);
+        rc = RegQueryValueEx(key->hkey, valuename, 0, resulttype,
+                             (LPBYTE)*result, (LPDWORD)resultsize);
         if (rc != ERROR_SUCCESS)
             return APR_FROM_OS_ERROR(rc);
     }
@@ -400,17 +400,17 @@ AP_DECLARE(apr_status_t) ap_regkey_value_raw_get(void **result,
 }
 
 
-AP_DECLARE(apr_status_t) ap_regkey_value_raw_set(ap_regkey_t *key, 
-                                                 const char *valuename, 
-                                                 const void *value, 
+AP_DECLARE(apr_status_t) ap_regkey_value_raw_set(ap_regkey_t *key,
+                                                 const char *valuename,
+                                                 const void *value,
                                                  apr_size_t valuesize,
                                                  apr_int32_t valuetype,
                                                  apr_pool_t *pool)
 {
     LONG rc;
-    
+
 #if APR_HAS_UNICODE_FS
-    IF_WIN_OS_IS_UNICODE 
+    IF_WIN_OS_IS_UNICODE
     {
         apr_size_t valuelen = strlen(valuename) + 1;
         apr_size_t wvallen = 256;
@@ -422,15 +422,15 @@ AP_DECLARE(apr_status_t) ap_regkey_value_raw_set(ap_regkey_t *key,
         else if (valuelen)
             return APR_ENAMETOOLONG;
 
-        rc = RegSetValueExW(key->hkey, wvalname, 0, valuetype, 
-                            (LPBYTE)value, valuesize);
+        rc = RegSetValueExW(key->hkey, wvalname, 0, valuetype,
+                            (LPBYTE)value, (DWORD)valuesize);
     }
 #endif /* APR_HAS_UNICODE_FS */
 #if APR_HAS_ANSI_FS
     ELSE_WIN_OS_IS_ANSI
     {
-        rc = RegSetValueEx(key->hkey, valuename, 0, valuetype, 
-                            (LPBYTE)value, valuesize);
+        rc = RegSetValueEx(key->hkey, valuename, 0, valuetype,
+                            (LPBYTE)value, (DWORD)valuesize);
     }
 #endif
     if (rc != ERROR_SUCCESS) {
@@ -440,9 +440,9 @@ AP_DECLARE(apr_status_t) ap_regkey_value_raw_set(ap_regkey_t *key,
 }
 
 
-AP_DECLARE(apr_status_t) ap_regkey_value_array_get(apr_array_header_t **result, 
+AP_DECLARE(apr_status_t) ap_regkey_value_array_get(apr_array_header_t **result,
                                                    ap_regkey_t *key,
-                                                   const char *valuename, 
+                                                   const char *valuename,
                                                    apr_pool_t *pool)
 {
     /* Retrieve a registry string value, and explode any envvars
@@ -453,7 +453,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_array_get(apr_array_header_t **result,
     char *buf;
     char *tmp;
     DWORD type;
-    DWORD size = 0;
+    apr_size_t size = 0;
 
     rv = ap_regkey_value_raw_get(&value, &size, &type, key, valuename, pool);
     if (rv != APR_SUCCESS) {
@@ -464,7 +464,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_array_get(apr_array_header_t **result,
     }
 
 #if APR_HAS_UNICODE_FS
-    IF_WIN_OS_IS_UNICODE 
+    IF_WIN_OS_IS_UNICODE
     {
         apr_size_t alloclen;
         apr_size_t valuelen = strlen(valuename) + 1;
@@ -490,7 +490,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_array_get(apr_array_header_t **result,
 #if APR_HAS_ANSI_FS
     ELSE_WIN_OS_IS_ANSI
     {
-        /* Small possiblity the array is either unterminated 
+        /* Small possiblity the array is either unterminated
          * or single NULL terminated.  Avert.
          */
         buf = (char *)value;
@@ -511,7 +511,7 @@ AP_DECLARE(apr_status_t) ap_regkey_value_array_get(apr_array_header_t **result,
         }
     }
 
-    *result = apr_array_make(pool, size, sizeof(char *));
+    *result = apr_array_make(pool, (int)size, sizeof(char *));
     for (tmp = buf; *tmp; ++tmp) {
         char **newelem = (char **) apr_array_push(*result);
         *newelem = tmp;
@@ -524,9 +524,9 @@ AP_DECLARE(apr_status_t) ap_regkey_value_array_get(apr_array_header_t **result,
 }
 
 
-AP_DECLARE(apr_status_t) ap_regkey_value_array_set(ap_regkey_t *key, 
-                                                   const char *valuename, 
-                                                   int nelts, 
+AP_DECLARE(apr_status_t) ap_regkey_value_array_set(ap_regkey_t *key,
+                                                   const char *valuename,
+                                                   int nelts,
                                                    const char * const * elts,
                                                    apr_pool_t *pool)
 {
@@ -536,9 +536,9 @@ AP_DECLARE(apr_status_t) ap_regkey_value_array_set(ap_regkey_t *key,
     int i;
     const void *value;
     apr_size_t bufsize;
-    
+
 #if APR_HAS_UNICODE_FS
-    IF_WIN_OS_IS_UNICODE 
+    IF_WIN_OS_IS_UNICODE
     {
         apr_status_t rv;
         apr_wchar_t *buf;
@@ -604,19 +604,19 @@ AP_DECLARE(apr_status_t) ap_regkey_value_array_set(ap_regkey_t *key,
         value = buf;
     }
 #endif
-    return ap_regkey_value_raw_set(key, valuename, value, 
+    return ap_regkey_value_raw_set(key, valuename, value,
                                    bufsize, REG_MULTI_SZ, pool);
 }
 
 
-AP_DECLARE(apr_status_t) ap_regkey_value_remove(const ap_regkey_t *key, 
+AP_DECLARE(apr_status_t) ap_regkey_value_remove(const ap_regkey_t *key,
                                                 const char *valuename,
                                                 apr_pool_t *pool)
 {
     LONG rc;
 
 #if APR_HAS_UNICODE_FS
-    IF_WIN_OS_IS_UNICODE 
+    IF_WIN_OS_IS_UNICODE
     {
         apr_size_t valuelen = strlen(valuename) + 1;
         apr_size_t wvallen = 256;
