@@ -1,9 +1,9 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 1999-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -38,6 +38,15 @@ AP_DECLARE_DATA module *ap_prelinked_modules[] = {
   NULL
 };
 
+ap_module_symbol_t ap_prelinked_module_symbols[] = {
+  {"core_module", &core_module},
+  {"win32_module", &win32_module},
+  {"mpm_winnt_module", &mpm_winnt_module},
+  {"http_module", &http_module},
+  {"so_module", &so_module},
+  {NULL, NULL}
+};
+
 AP_DECLARE_DATA module *ap_preloaded_modules[] = {
   &core_module,
   &win32_module,
