@@ -51,10 +51,6 @@
 #include "util_time.h"
 #include "ap_mpm.h"
 
-#ifndef APR_LARGEFILE
-#define APR_LARGEFILE 0
-#endif
-
 typedef struct {
     char    *t_name;
     int      t_val;
@@ -189,7 +185,7 @@ AP_DECLARE(void) ap_open_stderr_log(apr_pool_t *p)
     apr_file_open_stderr(&stderr_log, p);
 }
 
-AP_DECLARE(apr_status_t) ap_replace_stderr_log(apr_pool_t *p, 
+AP_DECLARE(apr_status_t) ap_replace_stderr_log(apr_pool_t *p,
                                                const char *fname)
 {
     apr_file_t *stderr_file;
@@ -241,6 +237,8 @@ static int log_child(apr_pool_t *p, const char *progname,
     apr_proc_t *procnew;
 
     if (((rc = apr_procattr_create(&procattr, p)) == APR_SUCCESS)
+        && ((rc = apr_procattr_cmdtype_set(procattr,
+                                           APR_SHELLCMD_ENV)) == APR_SUCCESS)
         && ((rc = apr_procattr_io_set(procattr,
                                       APR_FULL_BLOCK,
                                       APR_NO_PIPE,
@@ -330,7 +328,7 @@ static int open_error_log(server_rec *s, apr_pool_t *p)
     return OK;
 }
 
-int ap_open_logs(apr_pool_t *pconf, apr_pool_t *p /* plog */, 
+int ap_open_logs(apr_pool_t *pconf, apr_pool_t *p /* plog */,
                  apr_pool_t *ptemp, server_rec *s_main)
 {
     apr_status_t rc = APR_SUCCESS;
@@ -511,7 +509,14 @@ static void log_error_core(const char *file, int line, int level,
             }
             file = tmp;
         }
-#endif /*_OSD_POSIX*/
+#else /* _OSD_POSIX || WIN32 */
+        const char *p;
+        /* On Unix, __FILE__ may be an absolute path in a
+         * VPATH build. */
+        if (file[0] == '/' && (p = ap_strrchr_c(file, '/')) != NULL) {
+            file = p + 1;
+        }
+#endif /*_OSD_POSIX || WIN32 */
         len += apr_snprintf(errstr + len, MAX_STRING_LEN - len,
                             "%s(%d): ", file, line);
     }
@@ -674,7 +679,7 @@ AP_DECLARE(void) ap_log_pid(apr_pool_t *p, const char *filename)
 
     fname = ap_server_root_relative(p, filename);
     if (!fname) {
-        ap_log_error(APLOG_MARK, APLOG_STARTUP|APLOG_CRIT, APR_EBADPATH, 
+        ap_log_error(APLOG_MARK, APLOG_STARTUP|APLOG_CRIT, APR_EBADPATH,
                      NULL, "Invalid PID file path %s, ignoring.", filename);
         return;
     }
@@ -690,9 +695,9 @@ AP_DECLARE(void) ap_log_pid(apr_pool_t *p, const char *filename)
          *      based on the last modification time of the pid file.
          */
         ap_log_perror(APLOG_MARK, APLOG_WARNING, 0, p,
-                      apr_psprintf(p, "pid file %s overwritten -- Unclean "
-                                   "shutdown of previous Apache run?",
-                                   fname));
+                      "pid file %s overwritten -- Unclean "
+                      "shutdown of previous Apache run?",
+                      fname);
     }
 
     if ((rv = apr_file_open(&pid_file, fname,
@@ -727,7 +732,7 @@ AP_DECLARE(apr_status_t) ap_read_pid(apr_pool_t *p, const char *filename,
 
     fname = ap_server_root_relative(p, filename);
     if (!fname) {
-        ap_log_error(APLOG_MARK, APLOG_STARTUP|APLOG_CRIT, APR_EBADPATH, 
+        ap_log_error(APLOG_MARK, APLOG_STARTUP|APLOG_CRIT, APR_EBADPATH,
                      NULL, "Invalid PID file path %s, ignoring.", filename);
         return APR_EGENERAL;
     }
@@ -737,9 +742,7 @@ AP_DECLARE(apr_status_t) ap_read_pid(apr_pool_t *p, const char *filename,
         return rv;
     }
 
-    /* Ensure null-termination, so that strtol doesn't go crazy. */
     buf = apr_palloc(p, BUFFER_SIZE);
-    buf[BUFFER_SIZE - 1] = '\0';
 
     rv = apr_file_read_full(pid_file, buf, BUFFER_SIZE - 1, &bytes_read);
     if (rv != APR_SUCCESS && rv != APR_EOF) {
@@ -748,10 +751,11 @@ AP_DECLARE(apr_status_t) ap_read_pid(apr_pool_t *p, const char *filename,
 
     /* If we fill the buffer, we're probably reading a corrupt pid file.
      * To be nice, let's also ensure the first char is a digit. */
-    if (bytes_read == BUFFER_SIZE - 1 || !apr_isdigit(*buf)) {
+    if (bytes_read == 0 || bytes_read == BUFFER_SIZE - 1 || !apr_isdigit(*buf)) {
         return APR_EGENERAL;
     }
 
+    buf[bytes_read] = '\0';
     *mypid = strtol(buf, &endptr, 10);
 
     apr_file_close(pid_file);
@@ -789,6 +793,8 @@ static apr_status_t piped_log_spawn(piped_log *pl)
     apr_status_t status;
 
     if (((status = apr_procattr_create(&procattr, pl->p)) != APR_SUCCESS) ||
+        ((status = apr_procattr_cmdtype_set(procattr,
+                                            APR_SHELLCMD_ENV)) != APR_SUCCESS) ||
         ((status = apr_procattr_child_in_set(procattr,
                                              ap_piped_log_read_fd(pl),
                                              ap_piped_log_write_fd(pl)))
