@@ -1,9 +1,9 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 2001-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -15,9 +15,10 @@
  */
 
 /*
- * http_request.c: functions to get and process requests
+ * @file  request.c
+ * @brief functions to get and process requests
  *
- * Rob McCool 3/21/93
+ * @author Rob McCool 3/21/93
  *
  * Thoroughly revamped by rst for Apache.  NB this file reads
  * best from the bottom up.
@@ -355,8 +356,8 @@ static int resolve_symlink(char *d, apr_finfo_t *lfi, int opts, apr_pool_t *p)
     savename = (lfi->valid & APR_FINFO_NAME) ? lfi->name : NULL;
 
     if (opts & OPT_SYM_LINKS) {
-        if ((res = apr_stat(&fi, d, lfi->valid & ~(APR_FINFO_NAME 
-                                                 | APR_FINFO_LINK), p)) 
+        if ((res = apr_stat(&fi, d, lfi->valid & ~(APR_FINFO_NAME
+                                                 | APR_FINFO_LINK), p))
                  != APR_SUCCESS) {
             return HTTP_FORBIDDEN;
         }
@@ -376,7 +377,8 @@ static int resolve_symlink(char *d, apr_finfo_t *lfi, int opts, apr_pool_t *p)
      * owner of the symlink, then get the info of the target.
      */
     if (!(lfi->valid & APR_FINFO_OWNER)) {
-        if ((res = apr_lstat(&fi, d, lfi->valid | APR_FINFO_OWNER, p))
+        if ((res = apr_stat(&fi, d,
+                            lfi->valid | APR_FINFO_LINK | APR_FINFO_OWNER, p))
             != APR_SUCCESS) {
             return HTTP_FORBIDDEN;
         }
@@ -387,7 +389,7 @@ static int resolve_symlink(char *d, apr_finfo_t *lfi, int opts, apr_pool_t *p)
         return HTTP_FORBIDDEN;
     }
 
-    if (apr_compare_users(fi.user, lfi->user) != APR_SUCCESS) {
+    if (apr_uid_compare(fi.user, lfi->user) != APR_SUCCESS) {
         return HTTP_FORBIDDEN;
     }
 
@@ -418,6 +420,7 @@ typedef struct core_opts_t {
         allow_options_t add;
         allow_options_t remove;
         overrides_t override;
+        overrides_t override_opts;
 } core_opts_t;
 
 static void core_opts_merge(const ap_conf_vector_t *sec, core_opts_t *opts)
@@ -443,6 +446,7 @@ static void core_opts_merge(const ap_conf_vector_t *sec, core_opts_t *opts)
 
     if (!(this_dir->override & OR_UNSET)) {
         opts->override = this_dir->override;
+        opts->override_opts = this_dir->override_opts;
     }
 }
 
@@ -764,7 +768,7 @@ AP_DECLARE(int) ap_directory_walk(request_rec *r)
                     && ((entry_core->d_components < seg)
                      || (entry_core->d_is_fnmatch
                          ? (apr_fnmatch(entry_core->d, r->filename,
-                                        FNM_PATHNAME) != APR_SUCCESS)
+                                        APR_FNM_PATHNAME) != APR_SUCCESS)
                          : (strcmp(r->filename, entry_core->d) != 0)))) {
                     continue;
                 }
@@ -813,14 +817,15 @@ AP_DECLARE(int) ap_directory_walk(request_rec *r)
 
                 ap_conf_vector_t *htaccess_conf = NULL;
 
-                /* No htaccess in an incomplete root path, 
-                 * nor if it's disabled 
+                /* No htaccess in an incomplete root path,
+                 * nor if it's disabled
                  */
                 if (seg < startseg || !opts.override) {
                     break;
                 }
 
                 res = ap_parse_htaccess(&htaccess_conf, r, opts.override,
+                                        opts.override_opts,
                                         apr_pstrdup(r->pool, r->filename),
                                         sconf->access_name);
                 if (res) {
@@ -910,7 +915,7 @@ AP_DECLARE(int) ap_directory_walk(request_rec *r)
                 ++seg_name;
 
             /* If nothing remained but a '/' string, we are finished
-             */
+             * XXX: NO WE ARE NOT!!!  Now process this puppy!!! */
             if (!*seg_name) {
                 break;
             }
@@ -934,14 +939,16 @@ AP_DECLARE(int) ap_directory_walk(request_rec *r)
                 continue;
             }
 
-            /* We choose apr_lstat here, rather that apr_stat, so that we
-             * capture this path object rather than its target.  We will
-             * replace the info with our target's info below.  We especially
-             * want the name of this 'link' object, not the name of its
-             * target, if we are fixing the filename case/resolving aliases.
+            /* We choose apr_stat with flag APR_FINFO_LINK here, rather that
+             * plain apr_stat, so that we capture this path object rather than
+             * its target.  We will replace the info with our target's info
+             * below.  We especially want the name of this 'link' object, not
+             * the name of its target, if we are fixing the filename
+             * case/resolving aliases.
              */
-            rv = apr_lstat(&thisinfo, r->filename,
-                           APR_FINFO_MIN | APR_FINFO_NAME, r->pool);
+            rv = apr_stat(&thisinfo, r->filename,
+                          APR_FINFO_MIN | APR_FINFO_NAME | APR_FINFO_LINK,
+                          r->pool);
 
             if (APR_STATUS_IS_ENOENT(rv)) {
                 /* Nothing?  That could be nice.  But our directory
@@ -983,7 +990,8 @@ AP_DECLARE(int) ap_directory_walk(request_rec *r)
                 if ((res = resolve_symlink(r->filename, &thisinfo,
                                            opts.opts, r->pool)) != OK) {
                     ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
-                                  "Symbolic link not allowed: %s",
+                                  "Symbolic link not allowed "
+                                  "or link target not accessible: %s",
                                   r->filename);
                     return r->status = res;
                 }
@@ -991,7 +999,7 @@ AP_DECLARE(int) ap_directory_walk(request_rec *r)
 
             /* Ok, we are done with the link's info, test the real target
              */
-            if (thisinfo.filetype == APR_REG || 
+            if (thisinfo.filetype == APR_REG ||
                 thisinfo.filetype == APR_NOFILE) {
                 /* That was fun, nothing left for us here
                  */
@@ -1040,7 +1048,7 @@ AP_DECLARE(int) ap_directory_walk(request_rec *r)
                 continue;
             }
 
-            if (ap_regexec(entry_core->r, r->filename, 0, NULL, REG_NOTEOL)) {
+            if (ap_regexec(entry_core->r, r->filename, 0, NULL, AP_REG_NOTEOL)) {
                 continue;
             }
 
@@ -1231,7 +1239,7 @@ AP_DECLARE(int) ap_location_walk(request_rec *r)
             if (entry_core->r
                 ? ap_regexec(entry_core->r, r->uri, 0, NULL, 0)
                 : (entry_core->d_is_fnmatch
-                   ? apr_fnmatch(entry_core->d, cache->cached, FNM_PATHNAME)
+                   ? apr_fnmatch(entry_core->d, cache->cached, APR_FNM_PATHNAME)
                    : (strncmp(entry_core->d, cache->cached, len)
                       || (entry_core->d[len - 1] != '/'
                           && cache->cached[len] != '/'
@@ -1380,7 +1388,7 @@ AP_DECLARE(int) ap_file_walk(request_rec *r)
             if (entry_core->r
                 ? ap_regexec(entry_core->r, cache->cached , 0, NULL, 0)
                 : (entry_core->d_is_fnmatch
-                   ? apr_fnmatch(entry_core->d, cache->cached, FNM_PATHNAME)
+                   ? apr_fnmatch(entry_core->d, cache->cached, APR_FNM_PATHNAME)
                    : strcmp(entry_core->d, cache->cached))) {
                 continue;
             }
@@ -1523,6 +1531,11 @@ static request_rec *make_sub_request(const request_rec *r,
      */
     ap_run_create_request(rnew);
 
+    /* Begin by presuming any module can make its own path_info assumptions,
+     * until some module interjects and changes the value.
+     */
+    rnew->used_path_info = AP_REQ_DEFAULT_PATH_INFO;
+
     return rnew;
 }
 
@@ -1573,7 +1586,8 @@ AP_DECLARE(request_rec *) ap_sub_req_method_uri(const char *method,
                                                 ap_filter_t *next_filter)
 {
     request_rec *rnew;
-    int res;
+    /* Initialise res, to avoid a gcc warning */
+    int res = HTTP_INTERNAL_SERVER_ERROR;
     char *udir;
 
     rnew = make_sub_request(r, next_filter);
@@ -1598,17 +1612,25 @@ AP_DECLARE(request_rec *) ap_sub_req_method_uri(const char *method,
         return rnew;
     }
 
-    /* lookup_uri 
+    /* lookup_uri
      * If the content can be served by the quick_handler, we can
      * safely bypass request_internal processing.
+     *
+     * If next_filter is NULL we are expecting to be
+     * internal_fast_redirect'ed to the subrequest, or the subrequest will
+     * never be invoked. We need to make sure that the quickhandler is not
+     * invoked by any lookups. Since an internal_fast_redirect will always
+     * occur too late for the quickhandler to handle the request.
      */
-    res = ap_run_quick_handler(rnew, 1);
+    if (next_filter) {
+        res = ap_run_quick_handler(rnew, 1);
+    }
 
-    if (res != OK) {
+    if (next_filter == NULL || res != OK) {
         if ((res = ap_process_request_internal(rnew))) {
             rnew->status = res;
         }
-    } 
+    }
 
     return rnew;
 }
@@ -1677,10 +1699,10 @@ AP_DECLARE(request_rec *) ap_sub_req_lookup_dirent(const apr_finfo_t *dirent,
     if ((dirent->valid & APR_FINFO_MIN) != APR_FINFO_MIN) {
         /*
          * apr_dir_read isn't very complete on this platform, so
-         * we need another apr_lstat (or simply apr_stat if we allow
-         * all symlinks here.)  If this is an APR_LNK that resolves
-         * to an APR_DIR, then we will rerun everything anyways...
-         * this should be safe.
+         * we need another apr_stat (with or without APR_FINFO_LINK
+         * depending on whether we allow all symlinks here.)  If this
+         * is an APR_LNK that resolves to an APR_DIR, then we will rerun
+         * everything anyways... this should be safe.
          */
         apr_status_t rv;
         if (ap_allow_options(rnew) & OPT_SYM_LINKS) {
@@ -1691,8 +1713,9 @@ AP_DECLARE(request_rec *) ap_sub_req_lookup_dirent(const apr_finfo_t *dirent,
             }
         }
         else {
-            if (((rv = apr_lstat(&rnew->finfo, rnew->filename,
-                                 APR_FINFO_MIN, rnew->pool)) != APR_SUCCESS)
+            if (((rv = apr_stat(&rnew->finfo, rnew->filename,
+                                APR_FINFO_LINK | APR_FINFO_MIN,
+                                rnew->pool)) != APR_SUCCESS)
                 && (rv != APR_INCOMPLETE)) {
                 rnew->finfo.filetype = 0;
             }
@@ -1796,8 +1819,9 @@ AP_DECLARE(request_rec *) ap_sub_req_lookup_file(const char *new_file,
             }
         }
         else {
-            if (((rv = apr_lstat(&rnew->finfo, rnew->filename,
-                                 APR_FINFO_MIN, rnew->pool)) != APR_SUCCESS)
+            if (((rv = apr_stat(&rnew->finfo, rnew->filename,
+                                APR_FINFO_LINK | APR_FINFO_MIN,
+                                rnew->pool)) != APR_SUCCESS)
                 && (rv != APR_INCOMPLETE)) {
                 rnew->finfo.filetype = 0;
             }
@@ -1845,8 +1869,8 @@ AP_DECLARE(request_rec *) ap_sub_req_lookup_file(const char *new_file,
 AP_DECLARE(int) ap_run_sub_req(request_rec *r)
 {
     int retval = DECLINED;
-    /* Run the quick handler if the subrequest is not a dirent or file 
-     * subrequest 
+    /* Run the quick handler if the subrequest is not a dirent or file
+     * subrequest
      */
     if (!(r->filename && r->finfo.filetype)) {
         retval = ap_run_quick_handler(r, 0);
