@@ -1,4 +1,5 @@
-/* Copyright 1999-2004 The Apache Software Foundation
+/* Copyright 1999-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,9 +14,10 @@
  * limitations under the License.
  */
 
-/*
- * http_vhost.c: functions pertaining to virtual host addresses
- *      (configuration and run-time)
+/**
+ * @file  vhost.c
+ * @brief functions pertaining to virtual host addresses
+ *        (configuration and run-time)
  */
 
 #include "apr.h"
@@ -68,7 +70,7 @@ struct ipaddr_chain {
                                  * sharing this address */
 };
 
-/* This defines the size of the hash apr_table_t used for hashing ip addresses
+/* This defines the size of the hash table used for hashing ip addresses
  * of virtual hosts.  It must be a power of two.
  */
 #ifndef IPHASH_TABLE_SIZE
@@ -135,7 +137,7 @@ AP_DECLARE(void) ap_init_vhost_config(apr_pool_t *p)
  * port is the default port to assume
  */
 static const char *get_addresses(apr_pool_t *p, const char *w_,
-                                 server_addr_rec ***paddr, 
+                                 server_addr_rec ***paddr,
                                  apr_port_t default_port)
 {
     apr_sockaddr_t *my_addr;
@@ -181,18 +183,24 @@ static const char *get_addresses(apr_pool_t *p, const char *w_,
 
     if (strcmp(host, "*") == 0) {
         rv = apr_sockaddr_info_get(&my_addr, "0.0.0.0", APR_INET, port, 0, p);
-        ap_assert(rv == APR_SUCCESS); /* must be bug or out of storage */
+        if (rv) {
+            return "Could not resolve address '0.0.0.0' -- "
+                "check resolver configuration.";
+        }
     }
     else if (strcasecmp(host, "_default_") == 0
         || strcmp(host, "255.255.255.255") == 0) {
         rv = apr_sockaddr_info_get(&my_addr, "255.255.255.255", APR_INET, port, 0, p);
-        ap_assert(rv == APR_SUCCESS); /* must be bug or out of storage */
+        if (rv) {
+            return "Could not resolve address '255.255.255.255' -- "
+                "check resolver configuration.";
+        }
     }
     else {
         rv = apr_sockaddr_info_get(&my_addr, host, APR_UNSPEC, port, 0, p);
         if (rv != APR_SUCCESS) {
             ap_log_error(APLOG_MARK, APLOG_ERR, rv, NULL,
-                "Cannot resolve host name %s --- ignoring!", host);
+                "Could not resolve host name %s -- ignoring!", host);
             return NULL;
         }
     }
@@ -251,7 +259,7 @@ const char *ap_set_name_virtual_host(cmd_parms *cmd, void *dummy,
 }
 
 
-/* hash apr_table_t statistics, keep this in here for the beta period so
+/* hash table statistics, keep this in here for the beta period so
  * we can find out if the hash function is ok
  */
 #ifdef IPHASH_STATISTICS
@@ -364,7 +372,7 @@ static APR_INLINE ipaddr_chain *find_ipaddr(apr_sockaddr_t *sa)
     unsigned bucket;
     ipaddr_chain *trav;
 
-    /* scan the hash apr_table_t for an exact match first */
+    /* scan the hash table for an exact match first */
     bucket = hash_addr(sa);
     for (trav = iphash_table[bucket]; trav; trav = trav->next) {
         server_addr_rec *sar = trav->sar;
@@ -374,7 +382,7 @@ static APR_INLINE ipaddr_chain *find_ipaddr(apr_sockaddr_t *sa)
             if (apr_sockaddr_equal(cur, sa)) {
                 return trav;
             }
-        }    
+        }
     }
     return NULL;
 }
@@ -496,19 +504,13 @@ static void remove_unused_name_vhosts(server_rec *main_s, ipaddr_chain **pic)
 {
     while (*pic) {
         ipaddr_chain *ic = *pic;
-        
+
         if (ic->server == NULL) {
             ap_log_error(APLOG_MARK, APLOG_WARNING, 0, main_s,
                          "NameVirtualHost %s:%u has no VirtualHosts",
                          ic->sar->virthost, ic->sar->host_port);
             *pic = ic->next;
         }
-        else if (ic->names == NULL) {
-            /* if server != NULL and names == NULL then we're done
-             * looking at NameVirtualHosts
-             */
-            break;
-        }
         else {
             pic = &ic->next;
         }
@@ -539,7 +541,7 @@ AP_DECLARE(void) ap_fini_vhost_config(apr_pool_t *p, server_rec *main_s)
         iphash_table_tail[i] = &iphash_table[i];
     }
 
-    /* The first things to go into the hash apr_table_t are the NameVirtualHosts
+    /* The first things to go into the hash table are the NameVirtualHosts
      * Since name_vhost_list is in the same order that the directives
      * occured in the config file, we'll copy it in that order.
      */
@@ -548,7 +550,7 @@ AP_DECLARE(void) ap_fini_vhost_config(apr_pool_t *p, server_rec *main_s)
         unsigned bucket = hash_addr(sar->host_addr);
         ipaddr_chain *ic = new_ipaddr_chain(p, NULL, sar);
 
-        if (memcmp(sar->host_addr->ipaddr_ptr, inaddr_any, 
+        if (memcmp(sar->host_addr->ipaddr_ptr, inaddr_any,
                    sar->host_addr->ipaddr_len)) { /* not IN[6]ADDR_ANY */
             *iphash_table_tail[bucket] = ic;
             iphash_table_tail[bucket] = &ic->next;
@@ -566,7 +568,7 @@ AP_DECLARE(void) ap_fini_vhost_config(apr_pool_t *p, server_rec *main_s)
          */
     }
 
-    /* The next things to go into the hash apr_table_t are the virtual hosts
+    /* The next things to go into the hash table are the virtual hosts
      * themselves.  They're listed off of main_s->next in the reverse
      * order they occured in the config file, so we insert them at
      * the iphash_table_tail but don't advance the tail.
@@ -724,16 +726,16 @@ static void fix_hostname(request_rec *r)
     }
     else if (port) {
         /* Don't throw the Host: header's port number away:
-	   save it in parsed_uri -- ap_get_server_port() needs it! */
+           save it in parsed_uri -- ap_get_server_port() needs it! */
         /* @@@ XXX there should be a better way to pass the port.
-	 *         Like r->hostname, there should be a r->portno
-	 */
+         *         Like r->hostname, there should be a r->portno
+         */
         r->parsed_uri.port = port;
-	r->parsed_uri.port_str = apr_itoa(r->pool, (int)port);
+        r->parsed_uri.port_str = apr_itoa(r->pool, (int)port);
     }
 
-    /* if the hostname is an IPv6 numeric address string, it was validated 
-     * already; otherwise, further validation is needed 
+    /* if the hostname is an IPv6 numeric address string, it was validated
+     * already; otherwise, further validation is needed
      */
     if (r->hostname[0] != '[') {
         for (dst = host; *dst; dst++) {
@@ -819,7 +821,7 @@ AP_DECLARE(int) ap_matches_request_vhost(request_rec *r, const char *host,
 
     /* search all the <VirtualHost> values */
     /* XXX: If this is a NameVirtualHost then we may not be doing the Right Thing
-     * consider: 
+     * consider:
      *
      *     NameVirtualHost 10.1.1.1
      *     <VirtualHost 10.1.1.1>
@@ -873,7 +875,7 @@ static void check_hostalias(request_rec *r)
 
     last_s = NULL;
 
-    apr_sockaddr_port_get(&port, r->connection->local_addr);
+    port = r->connection->local_addr->port;
 
     /* Recall that the name_chain is a list of server_addr_recs, some of
      * whose ports may not match.  Also each server may appear more than
@@ -929,7 +931,7 @@ static void check_serverpath(request_rec *r)
     name_chain *src;
     apr_port_t port;
 
-    apr_sockaddr_port_get(&port, r->connection->local_addr);
+    port = r->connection->local_addr->port;
 
     /*
      * This is in conjunction with the ServerPath code in http_core, so we
@@ -982,6 +984,56 @@ AP_DECLARE(void) ap_update_vhost_from_headers(request_rec *r)
     }
 }
 
+/**
+ * For every virtual host on this connection, call func_cb.
+ */
+AP_DECLARE(int) ap_vhost_iterate_given_conn(conn_rec *conn,
+                                            ap_vhost_iterate_conn_cb func_cb,
+                                            void* baton)
+{
+    server_rec *s;
+    server_rec *last_s;
+    name_chain *src;
+    apr_port_t port;
+    int rv = 0;
+
+    if (conn->vhost_lookup_data) {
+        last_s = NULL;
+        port = conn->local_addr->port;
+
+        for (src = conn->vhost_lookup_data; src; src = src->next) {
+            server_addr_rec *sar;
+
+            /* We only consider addresses on the name_chain which have a
+             * matching port.
+             */
+            sar = src->sar;
+            if (sar->host_port != 0 && port != sar->host_port) {
+                continue;
+            }
+
+            s = src->server;
+
+            if (s == last_s) {
+                /* we've already done a callback for this vhost. */
+                continue;
+            }
+
+            last_s = s;
+
+            rv = func_cb(baton, conn, s);
+
+            if (rv != 0) {
+                break;
+            }
+        }
+    }
+    else {
+        rv = func_cb(baton, conn, conn->base_server);
+    }
+
+    return rv;
+}
 
 /* Called for a new connection which has a known local_addr.  Note that the
  * new connection is assumed to have conn->server == main server.
@@ -991,7 +1043,7 @@ AP_DECLARE(void) ap_update_vhost_given_ip(conn_rec *conn)
     ipaddr_chain *trav;
     apr_port_t port;
 
-    /* scan the hash apr_table_t for an exact match first */
+    /* scan the hash table for an exact match first */
     trav = find_ipaddr(conn->local_addr);
 
     if (trav) {
@@ -1004,7 +1056,7 @@ AP_DECLARE(void) ap_update_vhost_given_ip(conn_rec *conn)
     /* maybe there's a default server or wildcard name-based vhost
      * matching this port
      */
-    apr_sockaddr_port_get(&port, conn->local_addr);
+    port = conn->local_addr->port;
 
     trav = find_default_server(port);
     if (trav) {
