@@ -1,9 +1,9 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 1999-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -96,18 +96,15 @@ static int get_line(char *s, int n, apr_file_t *f)
     char ch;
     apr_status_t rv = APR_EINVAL;
 
-    /* we need 2 remaining bytes in buffer */
-    while (i < (n - 2) &&
+    while (i < (n - 1) &&
            ((rv = apr_file_getc(&ch, f)) == APR_SUCCESS) && (ch != '\n')) {
         s[i++] = ch;
     }
-    /* First remaining byte potentially used here */
     if (ch == '\n')
         s[i++] = ch;
-    /* Second remaining byte used here */
     s[i] = '\0';
 
-    if (rv != APR_SUCCESS) 
+    if (rv != APR_SUCCESS)
         return 1;
 
     return 0;
@@ -196,21 +193,21 @@ int main(int argc, const char * const argv[])
     char w[MAX_STRING_LEN];
     char x[MAX_STRING_LEN];
     int found;
-   
+
     apr_app_initialize(&argc, &argv, NULL);
-    atexit(terminate); 
+    atexit(terminate);
     apr_pool_create(&cntxt, NULL);
     apr_file_open_stderr(&errfile, cntxt);
 
 #if APR_CHARSET_EBCDIC
-    rv = apr_xlate_open(&to_ascii, "ISO8859-1", APR_DEFAULT_CHARSET, cntxt);
+    rv = apr_xlate_open(&to_ascii, "ISO-8859-1", APR_DEFAULT_CHARSET, cntxt);
     if (rv) {
         apr_file_printf(errfile, "apr_xlate_open(): %s (%d)\n",
                 apr_strerror(rv, line, sizeof(line)), rv);
         exit(1);
     }
 #endif
-    
+
     apr_signal(SIGINT, (void (*)(int)) interrupted);
     if (argc == 5) {
         if (strcmp(argv[1], "-c"))
@@ -225,7 +222,7 @@ int main(int argc, const char * const argv[])
                     apr_strerror(rv, errmsg, sizeof errmsg));
             exit(1);
         }
-        apr_file_printf(errfile, "Adding password for %s in realm %s.\n", 
+        apr_file_printf(errfile, "Adding password for %s in realm %s.\n",
                     argv[4], argv[3]);
         add_password(argv[4], argv[3], f);
         apr_file_close(f);
@@ -269,7 +266,7 @@ int main(int argc, const char * const argv[])
             continue;
         }
         else {
-            apr_file_printf(errfile, "Changing password for user %s in realm %s\n", 
+            apr_file_printf(errfile, "Changing password for user %s in realm %s\n",
                     user, realm);
             add_password(user, realm, tfp);
             found = 1;
@@ -285,7 +282,7 @@ int main(int argc, const char * const argv[])
      */
     if (apr_file_copy(dirname, argv[1], APR_FILE_SOURCE_PERMS, cntxt) !=
                 APR_SUCCESS) {
-        apr_file_printf(errfile, "%s: unable to update file %s\n", 
+        apr_file_printf(errfile, "%s: unable to update file %s\n",
                         argv[0], argv[1]);
     }
     apr_file_close(tfp);
