@@ -1,9 +1,9 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 1999-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -17,11 +17,11 @@
 /*
  * Simple program to rotate Apache logs without having to kill the server.
  *
- * Contributed by Ben Laurie <ben@algroup.co.uk>
+ * Contributed by Ben Laurie <ben algroup.co.uk>
  *
  * 12 Mar 1996
  *
- * Ported to APR by Mladen Turk <mturk@mappingsoft.com>
+ * Ported to APR by Mladen Turk <mturk mappingsoft.com>
  *
  * 23 Sep 2001
  *
@@ -171,7 +171,7 @@ int main (int argc, const char * const argv[])
             apr_finfo_t finfo;
             apr_off_t current_size = -1;
 
-            if ((nLogFD != NULL) && 
+            if ((nLogFD != NULL) &&
                 (apr_file_info_get(&finfo, APR_FINFO_SIZE, nLogFD) == APR_SUCCESS)) {
                 current_size = finfo.size;
             }
@@ -188,7 +188,7 @@ int main (int argc, const char * const argv[])
 
         if (nLogFD == NULL) {
             int tLogStart;
-                
+
             if (tRotation) {
                 tLogStart = (now / tRotation) * tRotation;
             }
@@ -208,8 +208,7 @@ int main (int argc, const char * const argv[])
                 sprintf(buf2, "%s.%010d", szLogRoot, tLogStart);
             }
             tLogEnd = tLogStart + tRotation;
-            apr_file_open(&nLogFD, buf2, 
-                          APR_APPEND | APR_WRITE | APR_CREATE | APR_LARGEFILE,
+            apr_file_open(&nLogFD, buf2, APR_READ | APR_WRITE | APR_CREATE | APR_APPEND,
                           APR_OS_DEFAULT, pool);
             if (nLogFD == NULL) {
                 /* Uh-oh. Failed to open the new log file. Try to clear
